/**
 * CForm
 * @author Tevin
 */

<template>
    <form
        class="c-form"
        @submit="evt=>handleSubmit()"
    >
        <slot :formData="formData" />
    </form>
</template>

<script>
import Taro from '@tarojs/taro';
import './cForm.scss';

export default {
    name: 'CForm',
    props: {
        formData: Object,
        onChange: Function,
        onFinish: Function,
    },
    data() {
        return {
            validators: {},
        };
    },
    methods: {
        handleSubmit() {
            const checklist = [];
            Object.keys(this.validators).forEach((key) => {
                checklist.push(this.validators[key]());
            });
            Promise.all(checklist).then((validations) => {
                for (let validation of validations) {
                    if (!validation.passed) {
                        Taro.showToast({
                            title: validation.msg,
                            icon: 'none',
                            mask: false,
                            duration: 2000,
                        });
                        return;
                    }
                }
                // 所有检查通过
                this.onFinish && this.onFinish();
            });
        },
    },
    mounted() {
        this.formData.$handleChange = (evt = []) => {
            Object.keys(evt).forEach((key) => {
                if (typeof this.formData[key] === 'undefined') {
                    this.$set(this.formData, key, evt[key]);
                } else {
                    this.formData[key] = evt[key];
                }
            });
            this.onChange && this.onChange(evt);
        };
        this.formData.$regItemValidator = (name, cb) => {
            this.validators[name] = cb;
        };
    },
    beforeDestroy() {
        this.validators = {};
    },
};
</script>